const http = require('http');
const path = require('path');
const yaml = require('node-yaml');
const fs = require('fs');
const mime = require('mime-types');
const { URL } = require('url');

function notFound(res){
    res.writeHead(404);
    res.end('Not Found\n');
}

function isFile(path){

    // Check for settings file.
    var s = false;
    try { s = fs.lstatSync(path) } catch(e) {
        return false;
    }

    // Check for directory.
    return !s.isDirectory();
}

module.exports = {
    start(conf){

        // Check for settings file.
        if(!isFile(conf)){
            console.log('Settings file was not found in: ' + conf);
            return false;
        }

        // Load settings and set base working dir.
        const cwd = path.dirname(conf);
        const settings = yaml.readSync(path.resolve(conf));

        // Check for required fields.
        if(!settings.port)
            console.log('Port must be specified in the settings.');
        if(!settings.sourceDir)
            console.log('Source Dir must be specified in the settings.');

        // Start HTTP server.
        const server = http.createServer((req, res) => {

            // Extracts the search path from URL.
            var u = new URL(req.url, 'http://127.0.0.1');
            var n = u.pathname;
            if(n.charAt(0) == '/')
                n = n.substr(1);
            if(n.charAt(n.length - 1) == '/')
                n = n.slice(0, -1);

            // Assemble full resource path.
            var p = path.resolve(cwd, settings.sourceDir, n);
            var type = mime.lookup(p) || 'application/octet-stream';

            // Check for requested file.
            if(!isFile(p)){
                notFound(res);
                return;
            }

            // Send resource to client.
            res.writeHead(200, { 'Content-Type': type, 'Access-Control-Allow-Origin': '*' });
            res.end(fs.readFileSync(p, null));
        });

        // Start and treat HTTP server.
        server.on('error', err => console.log("Error: ", err) );
        server.listen(settings.port, 'localhost');
        console.log('Server running at http://localhost:' + settings.port);

        // Return server instance to allow closing.
        return server;
    }
};
